﻿using Microsoft.SharePoint.Client;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace Mavention.SPSNL14.CustomActionsWeb {
    public partial class Default : System.Web.UI.Page {
        ClientContext clientContext;

        protected void Page_PreInit(object sender, EventArgs e) {
            Uri redirectUrl;
            switch (SharePointContextProvider.CheckRedirectionStatus(Context, out redirectUrl)) {
                case RedirectionStatus.Ok:
                    return;
                case RedirectionStatus.ShouldRedirect:
                    Response.Redirect(redirectUrl.AbsoluteUri, endResponse: true);
                    break;
                case RedirectionStatus.CanNotRedirect:
                    Response.Write("An error occurred while processing your request.");
                    Response.End();
                    break;
            }
        }

        protected void Page_Load(object sender, EventArgs e) {
            var spContext = SharePointContextProvider.Current.GetSharePointContext(Context);

            clientContext = spContext.CreateUserClientContextForSPHost();
        }

        #region ButtonHandlers
        protected void BreakSiteCollection_Click(object sender, EventArgs e) {
            RunDeploymentStep(DeployBreakSiteCollection, BreakSiteCollectionStatus);
        }

        protected void InstallGlobalScript_Click(object sender, EventArgs e) {
            RunDeploymentStep(DeployInstallGlobalScript, InstallGlobalScriptStatus);
        }

        protected void RemoveGlobalScript_Click(object sender, EventArgs e) {
            RunDeploymentStep(DeployRemoveGlobalScript, RemoveGlobalScriptStatus);
        }

        protected void InstallCustomAction_Click(object sender, EventArgs e) {
            RunDeploymentStep(DeployInstallCustomAction, InstallCustomActionStatus);
        }

        protected void RemoveCustomAction_Click(object sender, EventArgs e) {
            RunDeploymentStep(DeployRemoveCustomAction, RemoveCustomActionStatus);

        }

        private void RunDeploymentStep(Action<ClientContext> deploymentStep, System.Web.UI.WebControls.Label statusControl) {
            try {
                deploymentStep(clientContext);
                statusControl.Text = "DONE";
            }
            catch (Exception ex) {
                statusControl.Text = "ERROR";
                statusControl.ToolTip = ex.Message;
            }
        } 
        #endregion

        private void DeployBreakSiteCollection(ClientContext clientContext) {
            Site site = clientContext.Site;

            UserCustomAction customAction = site.UserCustomActions.Add();
            customAction.Location = "ScriptLink";
            customAction.ScriptSrc = String.Format("{0}://{1}/Scripts/HelloWorld.js", Request.Url.Scheme, Request.Url.Authority);
            customAction.Sequence = 1000;
            customAction.Update();

            clientContext.ExecuteQuery();
        }

        private void DeployInstallGlobalScript(ClientContext clientContext) {
            Site site = clientContext.Site;

            // Get List
            var lists = clientContext.LoadQuery(site.RootWeb.Lists.Where(l => l.RootFolder.Name == "Style Library"));
            clientContext.ExecuteQuery();
            List styleLibrary = lists.FirstOrDefault();

            if (styleLibrary != null) {
                // Check if file exists and check out if necessary
                clientContext.Load(site.RootWeb, w => w.ServerRelativeUrl);
                clientContext.ExecuteQuery();

                Microsoft.SharePoint.Client.File file = site.RootWeb.GetFileByServerRelativeUrl(String.Format("{0}/Style Library/HelloWorld.js", site.RootWeb.ServerRelativeUrl.TrimEnd('/')));
                clientContext.Load(file, f => f.Exists, f => f.CheckOutType);
                try {
                    clientContext.ExecuteQuery();

                    if (file.Exists &&
                        file.CheckOutType == CheckOutType.None) {
                        file.CheckOut();
                        clientContext.ExecuteQuery();
                    }
                }
                catch (ServerException ex) {
                    // expected if the file doesn't exist
                    if (ex.ServerErrorTypeName != "System.IO.FileNotFoundException") {
                        throw;
                    }
                }

                // Get File Contents
                string fileContents = System.IO.File.ReadAllText(Server.MapPath(@"..\Scripts\HelloWorld.js"));

                // Upload Script File
                file = styleLibrary.RootFolder.Files.Add(new FileCreationInformation {
                    Content = Encoding.UTF8.GetBytes(fileContents),
                    Overwrite = true,
                    Url = "HelloWorld.js"
                });
                file.CheckIn(String.Empty, CheckinType.MajorCheckIn);
                clientContext.ExecuteQuery();

                // Register Custom Action
                UserCustomAction customAction = site.UserCustomActions.Add();
                customAction.Location = "ScriptLink";
                customAction.ScriptSrc = "~SiteCollection/Style Library/HelloWorld.js";
                customAction.Sequence = 1000;
                customAction.Update();
                clientContext.ExecuteQuery();
            }
            else {
                throw new FileNotFoundException("'Style Library' not found");
            }
        }

        private void DeployRemoveGlobalScript(ClientContext clientContext) {
            Site site = clientContext.Site;

            // Delete Custom Actions
            var customActions = clientContext.LoadQuery(site.UserCustomActions.Where(a => a.ScriptSrc == "~SiteCollection/Style Library/HelloWorld.js"));
            clientContext.ExecuteQuery();

            foreach (UserCustomAction customAction in customActions) {
                customAction.DeleteObject();
            }

            // Execute only if Custom Actions have been found and removed
            if (clientContext.HasPendingRequest) {
                clientContext.ExecuteQuery();
            }

            // Delete File
            clientContext.Load(site.RootWeb, w => w.ServerRelativeUrl);
            clientContext.ExecuteQuery();

            Microsoft.SharePoint.Client.File file = site.RootWeb.GetFileByServerRelativeUrl(String.Format("{0}/Style Library/HelloWorld.js", site.RootWeb.ServerRelativeUrl.TrimEnd('/')));
            file.DeleteObject();
            clientContext.ExecuteQuery();
        }

        private void DeployInstallCustomAction(ClientContext clientContext) {
            Site site = clientContext.Site;

            // Get List
            var lists = clientContext.LoadQuery(site.RootWeb.Lists.Where(l => l.RootFolder.Name == "Style Library"));
            clientContext.ExecuteQuery();
            List styleLibrary = lists.FirstOrDefault();

            if (styleLibrary != null) {
                UploadScript("Mavention.SharePoint.InsertTOC.Loader.js", styleLibrary.RootFolder, clientContext);
                UploadScript("Mavention.SharePoint.InsertTOC.PageComponent.js", styleLibrary.RootFolder, clientContext);

                // Get Web Language
                clientContext.Load(site.RootWeb, w => w.Language);
                clientContext.ExecuteQuery();

                // Register Script Loader
                UserCustomAction pageComponentLoaderCustomAction = site.UserCustomActions.Add();
                pageComponentLoaderCustomAction.Location = "ScriptLink";
                pageComponentLoaderCustomAction.ScriptSrc = "~SiteCollection/Style Library/Mavention.SharePoint.InsertTOC.Loader.js";
                pageComponentLoaderCustomAction.Sequence = 1000;
                pageComponentLoaderCustomAction.Update();
                clientContext.ExecuteQuery();

                // Register Ribbon Button
                UserCustomAction ribbonButtonCustomAction = site.UserCustomActions.Add();
                ribbonButtonCustomAction.Location = "CommandUI.Ribbon";
                ribbonButtonCustomAction.Name = "MaventionInsertTOCButton";
                ribbonButtonCustomAction.CommandUIExtension = String.Format(Properties.Resources.RibbonXML, site.RootWeb.Language);
                ribbonButtonCustomAction.Sequence = 20;
                ribbonButtonCustomAction.Update();
                clientContext.ExecuteQuery();
            }
            else {
                throw new FileNotFoundException("'Style Library' not found");
            }
        }

        private void UploadScript(string fileName, Folder folder, ClientContext clientContext) {
            Microsoft.SharePoint.Client.File file = folder.Files.Add(new FileCreationInformation {
                Content = Encoding.UTF8.GetBytes(System.IO.File.ReadAllText(Server.MapPath(@"..\Scripts\" + fileName))),
                Overwrite = true,
                Url = fileName
            });
            file.CheckIn(String.Empty, CheckinType.MajorCheckIn);
            clientContext.ExecuteQuery();
        }

        private void DeployRemoveCustomAction(ClientContext clientContext) {
            Site site = clientContext.Site;

            // Delete Custom Actions
            var customActions = clientContext.LoadQuery(site.UserCustomActions.Where(a => a.ScriptSrc == "~SiteCollection/Style Library/Mavention.SharePoint.InsertTOC.Loader.js" || a.Name == "MaventionInsertTOCButton"));
            clientContext.ExecuteQuery();

            foreach (UserCustomAction customAction in customActions) {
                customAction.DeleteObject();
            }

            // Execute only if Custom Actions have been found and removed
            if (clientContext.HasPendingRequest) {
                clientContext.ExecuteQuery();
            }

            // Delete Files
            clientContext.Load(site.RootWeb, w => w.ServerRelativeUrl);
            clientContext.ExecuteQuery();

            Microsoft.SharePoint.Client.File loader = site.RootWeb.GetFileByServerRelativeUrl(String.Format("{0}/Style Library/Mavention.SharePoint.InsertTOC.Loader.js", site.RootWeb.ServerRelativeUrl.TrimEnd('/')));
            loader.DeleteObject();
            Microsoft.SharePoint.Client.File pageComponent = site.RootWeb.GetFileByServerRelativeUrl(String.Format("{0}/Style Library/Mavention.SharePoint.InsertTOC.PageComponent.js", site.RootWeb.ServerRelativeUrl.TrimEnd('/')));
            pageComponent.DeleteObject();
            clientContext.ExecuteQuery();
        }
    }
}